#!/usr/bin/env python

"""Setup script for the CDDB module distribution under Win32."""

__revision__ = "$Id: setup-win32.py,v 1.1 2001/03/10 22:34:03 che_fox Exp $"

from distutils.core import setup, Extension

setup (# Distribution meta-data
       name = "CDDB",
       version = "1.3",
       description = "Module for retrieving track information about audio CDs from CDDB",
       author = "Ben Gertzfield",
       author_email = "che@debian.org",
       url = "http://csl.cse.ucsc.edu/~ben/python/",

       # Description of the modules and packages in the distribution
       py_modules = ['CDDB', 'DiscID', 'win32/cdrom'],
       ext_modules = [ Extension('mci', ['win32/mci.c']) ],
#       data_files = [('', ['win32/mci.dll'])]
      )



#!/usr/bin/env python

# Module for fetching information about an audio compact disc and
# returning it in a format friendly to CDDB.

# If called from the command line, will print out disc info in a
# format identical to Robert Woodcock's 'cd-discid' program.

# Written 17 Nov 1999 by Ben Gertzfield <che@debian.org>
# This work is released under the GNU GPL, version 2 or later.

# Release version 1.3
# CVS ID: $Id: DiscID.py,v 1.9 2001/03/10 22:34:03 che_fox Exp $

import cdrom, sys

def cddb_sum(n):
    ret = 0
    
    while n > 0:
	ret = ret + (n % 10)
	n = n / 10

    return ret

def open(device=None, flags=None):
    # Allow this function to be called with no arguments,
    # specifying that we should call cdrom.open() with
    # no arguments.
    if device == None:
        return cdrom.open()
    elif flags == None:
        return cdrom.open(device)
    else:
        return cdrom.open(device, flags)

def disc_id(device):
    (first, last) = cdrom.toc_header(device)

    track_frames = []
    checksum = 0
    
    for i in range(first, last + 1):
	(min, sec, frame) = cdrom.toc_entry(device, i)
	checksum = checksum + cddb_sum(min*60 + sec)
	track_frames.append(min*60*75 + sec*75 + frame)

    (min, sec, frame) = cdrom.leadout(device)
    track_frames.append(min*60*75 + sec*75 + frame)

    total_time = (track_frames[-1] / 75) - (track_frames[0] / 75)
	       
    discid = ((checksum % 0xff) << 24 | total_time << 8 | last)

    return [discid, last] + track_frames[:-1] + [ track_frames[-1] / 75 ]

if __name__ == '__main__':

    dev_name = None
    device = None
    
    if len(sys.argv) >= 2:
	dev_name = sys.argv[1]

    if dev_name:
        device = open(dev_name)
    else:
        device = open()
        
    disc_info = disc_id(device)

    print ('%08lx' % disc_info[0]),

    for i in disc_info[1:]:
	print ('%d' % i),



#!/usr/bin/env python

# Module for retrieving CDDB v1 data from CDDB servers via HTTP

# Written 17 Nov 1999 by Ben Gertzfield <che@debian.org>
# This work is released under the GNU GPL, version 2 or later.

# Release version 1.3
# CVS ID: $Id: CDDB.py,v 1.7 2001/03/10 22:34:03 che_fox Exp $

import urllib, string, socket, os, struct, re

name = 'CDDB.py'
version = 1.3

if os.environ.has_key('EMAIL'):
    (default_user, hostname) = string.split(os.environ['EMAIL'], '@')
else:
    default_user = os.geteuid() or os.environ['USER'] or 'user'
    hostname = socket.gethostname() or 'host'

proto = 4
default_server = 'http://freedb.freedb.org/~cddb/cddb.cgi'

def query(track_info, server_url=default_server,
	  user=default_user, host=hostname, client_name=name,
          client_version=version):

    disc_id = track_info[0]
    num_tracks = track_info[1]

    query_str = (('%08lx %d ') % (disc_id, num_tracks))

    for i in track_info[2:]:
	query_str = query_str + ('%d ' % i)
	
    query_str = urllib.quote_plus(string.rstrip(query_str))

    url = "%s?cmd=cddb+query+%s&hello=%s+%s+%s+%s&proto=%i" % \
	  (server_url, query_str, user, host, client_name,
           client_version, proto)

    response = urllib.urlopen(url)
    
    # Four elements in header: status, category, disc-id, title
    header = string.split(string.rstrip(response.readline()), ' ', 3)

    header[0] = string.atoi(header[0])

    if header[0] == 200:		# OK
	result = { 'category': header[1], 'disc_id': header[2], 'title':
		   header[3] }

	return [ header[0], result ]

    elif header[0] == 211 or header[0] == 210: # multiple matches
	result = []

	for line in response.readlines():
	    line = string.rstrip(line)

	    if line == '.':		# end of matches
		break
					# otherwise:
					# split into 3 pieces, not 4
					# (thanks to bgp for the fix!)
	    match = string.split(line, ' ', 2)

	    result.append({ 'category': match[0], 'disc_id': match[1], 'title':
			    match[2] })

	return [ header[0], result ]

    else:
	return [ header[0], None ]

def read(category, disc_id, server_url=default_server, 
	 user=default_user, host=hostname, client_name=name,
         client_version=version):

    url = "%s?cmd=cddb+read+%s+%s&hello=%s+%s+%s+%s&proto=%i" % \
	  (server_url, category, disc_id, user, host, client_name,
           client_version, proto)

    response = urllib.urlopen(url)
    
    header = string.split(string.rstrip(response.readline()), ' ', 3)

    header[0] = string.atoi(header[0])
    if header[0] == 210 or header[0] == 417: # success or access denied
	reply = []

	for line in response.readlines():
	    line = string.rstrip(line)

	    if line == '.':
		break;

	    line = string.replace(line, r'\t', "\t")
	    line = string.replace(line, r'\n', "\n")
	    line = string.replace(line, r'\\', "\\")

	    reply.append(line)

	if header[0] == 210:		# success, parse the reply
	    return [ header[0], parse_read_reply(reply) ]
	else:				# access denied. :(
	    return [ header[0], reply ]
    else:
	return [ header[0], None ]

def parse_read_reply(comments):
    
    len_re = re.compile(r'#\s*Disc length:\s*(\d+)\s*seconds')
    revis_re = re.compile(r'#\s*Revision:\s*(\d+)')
    submit_re = re.compile(r'#\s*Submitted via:\s*(.+)')
    keyword_re = re.compile(r'([^=]+)=(.*)')

    result = {}

    for line in comments:
	keyword_match = keyword_re.match(line)
	if keyword_match:
	    (keyword, data) = keyword_match.groups()

	    if result.has_key(keyword):
		result[keyword] = result[keyword] + data
	    else:
		result[keyword] = data
	    continue

	len_match = len_re.match(line)
	if len_match:
	    result['disc_len'] = int(len_match.group(1))
	    continue

	revis_match = revis_re.match(line)
	if revis_match:
	    result['revision'] = int(revis_match.group(1))
	    continue

	submit_match = submit_re.match(line)
	if submit_match:
	    result['submitted_via'] = submit_match.group(1)
	    continue

    return result



#!/usr/bin/env python

# Example of how to use CDDB.py and DiscID.py

# Written 17 Nov 1999 by Ben Gertzfield <che@debian.org
# This work is released under the GNU GPL, version 2 or later.

# Release version 1.3

import DiscID, CDDB, sys, os

dev = None
cdrom = None

if len(sys.argv) >= 2:
    dev = sys.argv[1]

if dev:
    cdrom = DiscID.open(dev)
else:
    cdrom = DiscID.open()

print "Getting disc id in CDDB format...",

disc_id = DiscID.disc_id(cdrom)

print "Disc ID: %08lx Num tracks: %d" % (disc_id[0], disc_id[1])
print "Querying CDDB for info on disc...",

(query_stat, query_info) = CDDB.query(disc_id)

if query_stat == 200:
    print ("success!\nQuerying CDDB for track info of `%s'... " % 
	   query_info['title']),

    (read_stat, read_info) = CDDB.read(query_info['category'], 
				       query_info['disc_id'])
    if read_stat == 210:
	print "success!"
					# Start from 0, not 1
					# thanks to bgp for the fix!
	for i in range(0, disc_id[1]):
	    print "Track %.02d: %s" % (i+1, read_info['TTITLE' + `i`])
    else:
	print "failure getting track info, status: %i" % read_stat

elif query_stat == 210 or query_stat == 211:
    print "multiple matches found! Matches are:"
    for i in query_info:
	print "ID: %s Category: %s Title: %s" % \
	      (i['disc_id'], i['category'], i['title'])

else:
    print "failure getting disc info, status %i" % query_stat


	



#!/usr/bin/env python

# Supporting Ben Gertzfield's <che@debian.org> cdrom-Interface using mciSendString on WIN32
# Frank David <F.David@digitecgmbh.de>

# This work is released under the GNU GPL, version 2 or later.

import mci
import string

def toc_header(device):
   return (1, string.atoi(mci.mciSendString("status %s number of tracks" % device)))

def toc_entry(device, track):
   mci.mciSendString("set %s time format msf" % device)
   mciString = "status %s position track %i" % (device, track)
   resultStr = mci.mciSendString(mciString)
   result = string.split(resultStr, ':', 2)
   r = []
   for i in result: r.append(string.atoi(i))
   return r

def toc_entry_pos(device, track): 
   return toc_entry(device, track)
   

def toc_entry_len(device, track): 
   mci.mciSendString("set %s time format msf" % device)
   mciString = "status %s length track %i" % (device, track)
   resultStr = mci.mciSendString(mciString)
   result = string.split(resultStr, ':', 2)
   r = []
   for i in result: r.append(string.atoi(i))
   return r

def leadout(device):
   firstTrack, lastTrack = toc_header(device)
   trackPosMin, trackPosSecond, trackPosFrame = toc_entry_pos(device, lastTrack)
   trackLenMin, trackLenSecond, trackLenFrame = toc_entry_len(device, lastTrack)
   # calculate raw leadout
   leadoutMin, leadoutSecond, leadoutFrame = ( trackPosMin +  trackLenMin,  trackPosSecond  + trackLenSecond, trackPosFrame + trackLenFrame)   
   # add windows specific correction
   leadoutFrame = leadoutFrame + leadoutFrame
   # convert to minute, second, frame
   if leadoutFrame >= 75:
      leadoutFrame  = leadoutFrame - 75;
      leadoutSecond = leadoutSecond + 1           
   if leadoutSecond >= 60:
      leadoutSecond  = leadoutSecond - 60;
      leadoutMin = leadoutMin + 1           
   # return leadout as tuple
   return (leadoutMin, leadoutSecond, leadoutFrame)

def open(device="cdaudio", flags="wait"):
    mci.mciSendString("open %s %s" % (device, flags))
    return device



#!/usr/bin/env python

"""Setup script for the CDDB module distribution."""

__revision__ = "$Id: setup.py,v 1.4 2001/03/10 22:34:03 che_fox Exp $"

from distutils.core import setup, Extension

setup (# Distribution meta-data
       name = "CDDB",
       version = "1.3",
       description = "Module for retrieving track information about audio CDs from CDDB",
       author = "Ben Gertzfield",
       author_email = "che@debian.org",
       url = "http://csl.cse.ucsc.edu/~ben/python/",

       # Description of the modules and packages in the distribution
       py_modules = ['CDDB', 'DiscID'],
       ext_modules = [ Extension('cdrom', ['unix/cdrommodule.c']) ]
      )
